///* Copyright 2008 Daniel Vokó <vokoda@gmail.com>
// * 
// * This file is part of Sagittarius.
// *
// * Sagittarius is free software: you can redistribute it and/or modify
// * it under the terms of the GNU General Public License as published by
// * the Free Software Foundation, either version 2 of the License, or
// * (at your option) any later version.
// *
// * Sagittarius is distributed in the hope that it will be useful,
// * but WITHOUT ANY WARRANTY; without even the implied warranty of
// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// * GNU General Public License for more details.
// *
// * You should have received a copy of the GNU General Public License
// * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
// */

using System;
using Gdk;
using Gtk;
using Sagittarius.Metadata;
using Sagittarius.Exiftool;
using System.IO;
using System.Threading;

namespace Sagittarius
{
	public class PhotoInfo
	{
		private string filename;
		private Gdk.Pixbuf thumbnail;
		private Gdk.Pixbuf preview;
		private XmpMetadata metadata, newMetadata;
		private bool isPixbuf;
		private bool metadataChanged;
		private Config config;
		private CacheManager cManager;
		private string markup;
		private Gtk.ThreadNotify threadNotify;
		private Thread previewLoaderThread;
		private string fileSize;

		public PhotoInfo(string filename, Config config, CacheManager cManager)
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo constructor");
			this.filename=filename;
			this.config=config;
			this.cManager=cManager;
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo constructor");
		}
		
		public void WriteMetadata()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.WriteMetadata");
			newMetadata.WriteMetadata(config.CreateBackup);
			metadata=newMetadata.Copy();
			cManager.UpdateCacheDate(filename, new FileInfo(filename).LastWriteTime.ToString());
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.WriteMetadata");
		}
		
		public void DiscardMetadataChanges ()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.DiscardMetadataChanges");
			newMetadata = metadata.Copy();
			metadataChanged = false;
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.DiscardMetadataChanges");
		}

		public Pixbuf GetPreview(int boxWidth, int boxHeight)
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.GetPreview");
			int previewWidth,previewHeight;
			double boxProp = Convert.ToDouble(boxWidth)/Convert.ToDouble(boxHeight);
			//Console.Write(" checking if the preview is already extracted or not... ");
			if(preview==null)
			{
				//Console.WriteLine(" NOT EXTRACTED");
				preview = new Pixbuf(null,"null.png");
				ThreadStart tStart = new ThreadStart (LoadPreview);
				this.previewLoaderThread = new Thread (tStart);
				this.previewLoaderThread.Start ();
				threadNotify = new Gtk.ThreadNotify (new Gtk.ReadyEvent (LoadPreviewFinished));

			}
			double prop=(double)preview.Width/(double)preview.Height;
			
			if (prop>=boxProp)
			{
				// a kép szélesebb, szélességhez kell igazítani
				previewWidth = boxWidth;
				previewHeight = Convert.ToInt32(preview.Height*Convert.ToDouble(boxWidth)/Convert.ToDouble(preview.Width));
			}
			else
			{
				previewHeight = boxHeight;
				previewWidth = Convert.ToInt32(preview.Width*Convert.ToDouble(boxHeight)/Convert.ToDouble(preview.Height));
			}
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.GetPreview");
			return preview.ScaleSimple(previewWidth,previewHeight,InterpType.Bilinear);
		}
		
		private void LoadPreview()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.LoadPreview");
			if(isPixbuf)
			{
				using (Pixbuf pixbuf = new Pixbuf (filename)) 
				{
					GeneratePreview(pixbuf);
				}
			}
			else
			{
				ExiftoolWrapper exiftoolWrapper = new ExiftoolWrapper(config.ExitoolExecutable);
				using(Pixbuf pixbuf = new Pixbuf((System.IO.Stream)exiftoolWrapper.GetBinaryTagFromFile(filename,"preview")))
				{
					GeneratePreview(pixbuf);
				}
				exiftoolWrapper = null;
			}
			cManager.AddPreviewCache(filename,preview);
			threadNotify.WakeupMain ();
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.LoadPreview");
		}
		
		private void LoadPreviewFinished()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.LoadPreviewFinished");
			if (OnPreviewLoaded != null)
			{
				OnPreviewLoaded (this, new PreviewLoadedEventArgs (Preview,this.Filename));
			}
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.LoadPreviewFinished");
		}

		private void GeneratePreview(Pixbuf pixbuf)
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.GeneratePreview");
			int photoWidth= pixbuf.Width;
			int photoHeight= pixbuf.Height;
			double prop=(double)photoHeight/(double)photoWidth;
			
			if((photoWidth>config.PrevMaxSize)||(photoHeight>config.PrevMaxSize))
			{
				if(photoWidth>=photoHeight)
				{
					preview = pixbuf.ScaleSimple(config.PrevMaxSize,(int)(prop*config.PrevMaxSize),InterpType.Bilinear);
				}
				else
				{
					preview = pixbuf.ScaleSimple((int)(config.PrevMaxSize/prop),config.PrevMaxSize,InterpType.Bilinear);
				}
			}
			else
			{
				preview = pixbuf.Copy();
			}
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.GeneratePreview");
		}
		
		private void CreateMarkup()
		{
			//Console.WriteLine("Entering Sagittarius.PhotoInfo.CreateMarkup");
			markup=
				"Filename: <b>"+this.ShortFilename+"</b>\n"+
				"File size: <b>"+FileSize+"</b>\n"+
				"Image dimensions: <b>no info</b>\n"+
				"Image caption: <b>"+Metadata.XmpIptcSection.GetValue(ExiftoolTags.IPTC_DESCRIPTION)+"</b>";
			//Console.WriteLine("Leaving Sagittarius.PhotoInfo.CreateMarkup");
		}
		
		
		public string Markup
		{
			get 
			{
				//Console.WriteLine("Entering Sagittarius.PhotoInfo.Markup");
				//Console.WriteLine("markup"+markup);
				if (markup== null)
				{
					CreateMarkup();
				}
				return markup; 
			}
		}

		public string FileSize
		{
			get
			{
				if(fileSize==null)
				{
					long size= new FileInfo(Filename).Length;
					if(size<1000) 
					{
						fileSize= size.ToString() + " byte";
					}
					else if(size>=1000) 
					{
						fileSize= Math.Abs(size/1000).ToString() + " Kb";
					}
				}
				return fileSize;
			}
		}

		public string Filename
		{
			get { return filename; }
			set { filename = value; }
		}
		
		public Gdk.Pixbuf Thumbnail
		{
			get { return thumbnail; }
			set { thumbnail = value; }
		}
		
		public Gdk.Pixbuf Preview
		{
			get { return preview; }
			set { preview = value; }
		}
		
		public XmpMetadata Metadata
		{
			get { return newMetadata; }
			set 
			{
				metadata= value;
				newMetadata = metadata.Copy();
			}
		}
		
		public bool IsPixbuf
		{
			get { return isPixbuf; }
			set { isPixbuf = value; }
		}
		
		public bool MetadataChanged
		{
			get { return metadataChanged; }
			set { metadataChanged = value; }
		}
		public string ShortFilename
		{
			get
			{
				int pos = Filename.LastIndexOf("/");
				return Filename.Substring(pos+1,filename.Length-pos-1);
			}
		}

		public delegate void OnPreviewLoadedHandler (PhotoInfo o, PreviewLoadedEventArgs args);
		public event OnPreviewLoadedHandler OnPreviewLoaded;

	}
}
